package eu.osci.messageparts;

import java.lang.ref.SoftReference;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;


/**
 * Class which holds JAXB Marshaller and Unmarshaller in a pool.<br>
 * Copyright  2010 Freie Hansestadt Bremen. <br>
 * Created by bremen online services Entwicklungs- und Betriebsgesellschaft mbH & Co. KG. <br>
 * This library (Starter Kit) can be used by anybody according to the Bremer Lizenz fr freie
 * Softwarebibliotheken. The license terms can be viewed at www.osci.de or can be ordered at the
 * OSCI-Leitstelle (Die Senatorin fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40, 28015 Bremen).
 * 
 * @author W. Oley
 */
public class JaxbMarshallerHolder
{

  protected JAXBContext context;

  private static JaxbMarshallerHolder osciMarshallerHolder = null;

  private static JaxbMarshallerHolder addressingMarshallerHolder = null;

  private static JaxbMarshallerHolder xkmsMarshallerHolder = null;

  private static JaxbMarshallerHolder letterStyleMarshallerHolder = null;

  private static JaxbMarshallerHolder wsTrustMarshallerHolder = null;

  private final List<SoftReference<Marshaller>> marshallers = new ArrayList<SoftReference<Marshaller>>();

  private final List<SoftReference<Unmarshaller>> unmarshallers = new ArrayList<SoftReference<Unmarshaller>>();

  /**
   * Constructor to create a JaxbMarshallerHolder object with the given JAXB context.
   * 
   * @param context JAXBContext
   * @throws JAXBException
   */
  public JaxbMarshallerHolder(JAXBContext context) throws JAXBException
  {
    this.context = context;
  }

  /**
   * Returns a Marshaller object matching the JAXB context managed by this object. After using it, you should
   * give it back by calling {@link #recycle(Marshaller)}.
   * 
   * @throws JAXBException
   */
  public Marshaller getMarshaller() throws JAXBException
  {
    synchronized (marshallers)
    {
      while (!marshallers.isEmpty())
      {
        SoftReference<Marshaller> ref = marshallers.remove(0);
        if (ref.get() != null)
        {
          return ref.get();
        }
      }
    }
    return context.createMarshaller();
  }

  /**
   * Recycles a {@link Marshaller} object back to be used again later.
   * 
   * @param marshaller to be recycled
   */
  public void recycle(Marshaller marshaller)
  {
    synchronized (marshallers)
    {
      SoftReference<Marshaller> ref = new SoftReference<Marshaller>(marshaller);
      marshallers.add(ref);
    }
  }

  /**
   * Returns an Unmarshaller object matching the JAXB context managed by this object. After using it, you
   * should give it back by calling {@link #recycle(Unmarshaller)}.
   * 
   * @throws JAXBException
   */
  public Unmarshaller getUnmarshaller() throws JAXBException
  {
    synchronized (unmarshallers)
    {
      while (!unmarshallers.isEmpty())
      {
        SoftReference<Unmarshaller> ref = unmarshallers.remove(0);
        if (ref.get() != null)
        {
          return ref.get();
        }
      }
    }
    return context.createUnmarshaller();
  }

  /**
   * Recycles an {@link Unmarshaller} object back to be used again later.
   * 
   * @param unmarshaller to be recycled
   */
  public void recycle(Unmarshaller unmarshaller)
  {
    synchronized (unmarshallers)
    {
      SoftReference<Unmarshaller> ref = new SoftReference<Unmarshaller>(unmarshaller);
      unmarshallers.add(ref);
    }
  }

  /**
   * Returns a Marshaller for the OSCI context for performance reasons.
   * 
   * @throws JAXBException
   */
  public static JaxbMarshallerHolder getOSCIMarshallerHolder() throws JAXBException
  {
    if (JaxbMarshallerHolder.osciMarshallerHolder == null)
    {
      osciMarshallerHolder = new JaxbMarshallerHolder(JaxbContextHolder.getOSCIJaxBContext());
    }
    return osciMarshallerHolder;
  }

  /**
   * Returns a Marshaller for the addressing context for performance reasons.
   * 
   * @throws JAXBException
   */
  public static JaxbMarshallerHolder getAddressingMarshallerHolder() throws JAXBException
  {
    if (JaxbMarshallerHolder.addressingMarshallerHolder == null)
    {
      addressingMarshallerHolder = new JaxbMarshallerHolder(JaxbContextHolder.getAddressingJaxBContext());
    }
    return addressingMarshallerHolder;
  }

  /**
   * Returns a Marshaller for the XKMS context for performance reasons.
   * 
   * @throws JAXBException
   */
  public static JaxbMarshallerHolder getXKMSMarshallerHolder() throws JAXBException
  {
    if (JaxbMarshallerHolder.xkmsMarshallerHolder == null)
    {
      xkmsMarshallerHolder = new JaxbMarshallerHolder(JaxbContextHolder.getXKMSJaxBContext());
    }
    return xkmsMarshallerHolder;
  }

  /**
   * Returns a Marshaller for the LetterStyle context for performance reasons.
   * 
   * @throws JAXBException
   */
  public static JaxbMarshallerHolder getLetterStyleMarshallerHolder() throws JAXBException
  {
    if (JaxbMarshallerHolder.letterStyleMarshallerHolder == null)
    {
      letterStyleMarshallerHolder = new JaxbMarshallerHolder(JaxbContextHolder.getLetterStyleJaxBContext());
    }
    return letterStyleMarshallerHolder;
  }

  /**
   * Returns a Marshaller for the WSTrust context for performance reasons.
   * 
   * @throws JAXBException
   */
  public static JaxbMarshallerHolder getWsTrusteMarshallerHolder() throws JAXBException
  {
    if (JaxbMarshallerHolder.wsTrustMarshallerHolder == null)
    {
      wsTrustMarshallerHolder = new JaxbMarshallerHolder(JaxbContextHolder.getTrustJaxBContext());
    }
    return wsTrustMarshallerHolder;
  }

}
