package eu.osci.messagetypes;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.ByteArrayOutputStream;
import java.math.BigInteger;
import java.net.URL;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;

import org.junit.Test;

import eu.osci.messageparts.OSCIFragmentCreator;


/**
 * Test class for the {@link OSCIMsgBoxFetchRequest}.<br>
 * Tests all the different fetch selectors from the OSCIMsgBoxFetchRequest class and some important header
 * elements. To run this tests an implementation must be installed under the URL declared in the static field
 * MessageParent.URL_MSGBOX_HOST. The installation of the msgBox must provide WSDLs under the URL
 * MessageParent.URL_WSDL_USER, MessageParent.URL_WSDL_Synchron, MessageParent.URL_WSDL_GERNERIC_USER,
 * MessageParent.WSA_REPLY_TO.<br>
 * Copyright  2010 Freie Hansestadt Bremen. <br>
 * Created by bremen online services Entwicklungs- und Betriebsgesellschaft mbH & Co. KG. <br>
 * This library (Starter Kit) can be used by anybody according to the Bremer Lizenz fr freie
 * Softwarebibliotheken. The license terms can be viewed at www.osci.de or can be ordered at the
 * OSCI-Leitstelle (Die Senatorin fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40, 28015 Bremen).
 * 
 * @author R. Lindemann
 */
// TODO > Version 1.0.1 for the messageId and relatesTo test must be a test implemented with multiple id.
// TODO implement a test with the try to commit a messageId, which was not fetched from the iterator
public class TestOSCIMsgBoxFetchRequest extends MessageParent
{

  private static Logger LOG = Logger.getLogger(TestOSCIMsgBoxFetchRequest.class.getName());

  public static final QName QNAME_PORT_USER = new QName("http://messagebox.osci20.bos-bremen.de",
                                                        "LetterStyle");

  public static QName QNAME_LETTER_STYLE_ROOT_ELEM = new QName(
                                                               "http://www.osci.eu/2008/05/common/urn/messageTypes/LetterStyle",
                                                               "LetterStyleMessage");

  /**
   * Fetches all new OSCI messages. At first there will be sent three OSCIRequest messages to prepare the
   * test.<br>
   * To fetch the OSCI messages, there will be used an OSCIMsgBoxFetchRequest and then an
   * OSCIMsgBoxGetNextRequest. Finally an OSCIMsgBoxCloseRequest will be used to finish the transaction.<br>
   * Expected behaviour: Successful sending of the OSCI message without any exceptions. The items pending
   * element will be checked in all steps.
   */
  @Test
  public void testFetchAll()
  {
    LOG.fine("Start testFetchAll");
    try
    {
      MessageParent.sendToMsgBox(true);
      MessageParent.sendToMsgBox(true);
      MessageParent.sendToMsgBox(true);
      OSCIMsgBoxFetchRequest req = new OSCIMsgBoxFetchRequest(new URL(URL_WSDL_USER_FETCH), SERVICE,
                                                              FETCH_PORT);
      req.setSelectionAll(true);
      OSCIMsgBoxFetchResponse rsp = req.sendRequest();

      LOG.info("request.messageId: " + req.getMessageId().getValue());
      LOG.info("response.messageId: " + rsp.getMessageId().getValue());

      assertFalse("No message found for selection", rsp.noMessageFoundForSelection());
      assertTrue("Wrong count of pending items found: "
                 + rsp.getMsgBoxResponse().getItemsPending().intValue(), rsp.getMsgBoxResponse()
                                                                            .getItemsPending()
                                                                            .intValue() >= 2);
      assertNotNull("No request iterator.", rsp.getMsgBoxResponse().getMsgBoxRequestID());
      assertTrue("No request iterator.", rsp.getMsgBoxResponse().getMsgBoxRequestID().length() > 0);
      String firstMsgId = rsp.getReceiveMessage().getMessageId().getValue();
      checkBody(rsp);
      // ########## Fetch next message ###########
      OSCIMsgBoxGetNextRequest nextReq = new OSCIMsgBoxGetNextRequest(req);

      rsp = nextReq.sendFetchRequest(new String[]{rsp.getReceiveMessage().getMessageId().getValue()});

      LOG.info("nextreq.messageId: " + nextReq.getMessageId().getValue());
      LOG.info("response.messageId: " + rsp.getMessageId().getValue());
      assertFalse("Wrong MessageId received.", firstMsgId.equals(rsp.getReceiveMessage()
                                                                    .getMessageId()
                                                                    .getValue()));

      assertFalse("No message found for selection", rsp.noMessageFoundForSelection());
      assertTrue("Wrong count of pending items found: "
                 + rsp.getMsgBoxResponse().getItemsPending().intValue(), rsp.getMsgBoxResponse()
                                                                            .getItemsPending()
                                                                            .intValue() >= 1);
      assertNotNull("No request iterator.", rsp.getMsgBoxResponse().getMsgBoxRequestID());
      assertTrue("No request iterator.", rsp.getMsgBoxResponse().getMsgBoxRequestID().length() > 0);
      checkBody(rsp);
      // ##########now sending the close msg. ################
      OSCIMsgBoxCloseRequest close = new OSCIMsgBoxCloseRequest(nextReq);
      close.sendRequest(new String[]{rsp.getReceiveMessage().getMessageId().getValue()});
    }
    catch (Exception ex)
    {
      LOG.log(Level.SEVERE, "Error: ", ex);
      fail("Sending testFetchAll request failed. Error: " + ex.getMessage());
    }
  }

  /**
   * Fetches an OSCI message with the selection for messageID. At first there will be sent an OSCI message
   * that will be fetched in this test.<br>
   * Expected behaviour: Successful sending of the OSCI message without any exceptions.
   */
  @Test
  public void testMessageId()
  {
    LOG.fine("Start testMessageId");
    try
    {
      OSCIMessage req = MessageParent.sendToMsgBox(true);
      OSCIMsgBoxFetchRequest reqFetch = new OSCIMsgBoxFetchRequest(new URL(URL_WSDL_USER_FETCH), SERVICE,
                                                                   FETCH_PORT);
      reqFetch.setSelectionByMessageId(new String[]{req.getMessageId().getValue()});
      OSCIMsgBoxFetchResponse rsp = reqFetch.sendRequest();
      assertFalse("No message found for selection", rsp.noMessageFoundForSelection());
      assertEquals("Wrong status list count: ", BigInteger.valueOf(0), rsp.getMsgBoxResponse()
                                                                          .getItemsPending());
      assertNotNull("No request iterator.", rsp.getMsgBoxResponse().getMsgBoxRequestID());
      assertTrue("No request iterator.", rsp.getMsgBoxResponse().getMsgBoxRequestID().length() > 0);
      checkBody(rsp);
    }
    catch (Exception ex)
    {
      LOG.log(Level.SEVERE, "Error: ", ex);
      // EndpointReferenceType fetchRef = OSCIFragmentCreator.createEndpointRef(URL_WSDL_USER);
      fail("Sending testMessageId request failed. Error: " + ex.getMessage());
    }
  }

  /**
   * Tests the case that there are no OSCI messages available for the selection. No OSCI message will send
   * before and the selection will be 'by date'.<br>
   * Expected behaviour: Successful sending of the OSCI message without any exceptions. In the
   * OSCIMsgBoxFetchResponse the NoMessageAvailable element is present.
   */
  @Test
  public void testNoMessageAvailable()
  {
    LOG.fine("Start testNoMessageAvailable");
    try
    {
      OSCIMsgBoxFetchRequest reqFetch = new OSCIMsgBoxFetchRequest(new URL(URL_WSDL_USER_FETCH), SERVICE,
                                                                   FETCH_PORT);
      reqFetch.setSelectionByDate(new Date(), new Date(), true);
      OSCIMsgBoxFetchResponse rsp = reqFetch.sendRequest();
      assertTrue("No message found for selection", rsp.noMessageFoundForSelection());
      assertEquals("Unexpected request iterator. Did not found 'NoElementIterator'",
                   "NoElementIterator",
                   rsp.getMsgBoxResponse().getMsgBoxRequestID());
      assertNotNull("MsgBoxResponseType is null", rsp.getMsgBoxResponse());
      assertNotNull("NoMessageAvailable is null", rsp.getMsgBoxResponse().getNoMessageAvailable());
      assertEquals("Wrong reason in NoMessageAvailable. ",
                   "NoSelectionFoundForMessageRequest",
                   rsp.getMsgBoxResponse().getNoMessageAvailable().getReason());
    }
    catch (Exception ex)
    {
      LOG.log(Level.SEVERE, "Error: ", ex);
      fail("Sending testNoMessageAvailable request failed. Error: " + ex.getMessage());
    }
  }

  private void checkBody(OSCIMsgBoxFetchResponse rsp) throws TransformerException
  {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    rsp.getReceiveMessageStream(out);
    assertTrue("Expected body not available.", out.toByteArray().length > 100);
    LOG.fine("The body: " + out);
  }

  /**
   * Configures the addressing header fields and the OSCI header fields of an OSCI message. <br>
   * Expected behaviour: Successful sending of the OSCI message without any exceptions. The changed addressing
   * header should be the correct values.
   * 
   * @throws Exception Should be thrown in all error cases.
   */
  @Test
  public void testFields() throws Exception
  {
    LOG.fine("Start testAddressingFields");

    OSCIRequest sendReq = (OSCIRequest)sendToMsgBox(true);
    String msgId = sendReq.getMessageId().getValue();
    // String faultTo = "http://myFaultURI.com";
    String from = "http://myFromURI.com";
    String replyURL = "http://replyURL.de";

    OSCIMsgBoxFetchRequest req = new OSCIMsgBoxFetchRequest(new URL(URL_WSDL_USER_FETCH), SERVICE, FETCH_PORT);
    req.setSelectionByMessageId(new String[]{msgId});
    // Test all header elements
    // should better not set
    // req.setAction("")
    req.setFrom(OSCIFragmentCreator.createEndpointRef(from));
    // should better not set
    // req.setMessageId(_messageId);
    // should better not set
    // req.setReplyTo(OSCIFragmentCreator.createEndpointRef(replyURL));

    // should better not set
    // req.setTo(_to);
    // should better not set
    // req.setTypeOfBusinessScenario(null);
    req.sendRequest();
    assertEquals("AdditionalHeader not 0", 0, req.getAdditionalHeaderList().size());

    assertNotNull("No From header found.", req.getFrom());
    assertEquals("Wrong url in From header.", from, req.getFrom().getAddress().getValue());
    assertNotNull("No TypeOfBus con.", req.getTypeOfBusinessScenario());
    assertNotNull("Wrong addressing entry for 'Action' element.", req.getAction().getValue());
    assertNotNull("Wrong addressing entry for 'MessageID' element.", req.getMessageId().getValue());
    LOG.fine("ReplyTo: " + req.getReplyTo().getAddress().getValue());
    assertNotSame("Wrong addressing entry for 'ReplyTo' element.", replyURL, req.getReplyTo()
                                                                                .getAddress()
                                                                                .getValue());
    assertNotNull("Wrong addressing entry for 'To' element.", req.getTo().getValue());
  }

}
